#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os
import pwd
import datetime
import ldap
import commands
import socket
import logging
import grp

from crypt import crypt
from PyQt4 import QtGui, QtCore
from PyQt4.QtGui import *

from cgauserconfig import CUSTOMUSERSFILE
from cgauserconfig import DEFAULTGROUPS



class MissingUserData(Exception): pass
class CgaUserExists(Exception): pass
class CgaUserCreationError(Exception): pass


class CgaUser:

    def __init__(self, login, nombre, passwd):
        self.id, self.nombre, self.password = login, nombre, passwd


    def save(self):
        self._save()


    def _save(self):
        self._add_to_system()
        self._add_to_list()


    def _get_new_uid(self):
        validos = [ t.pw_uid for t in pwd.getpwall() if 1000 <= t.pw_uid < 10000 ]
        validos.sort()
        return validos[-1]+1        


    def _add_to_system(self):
        # Se crea el usuario
        # Primero se cifra el password para pasarlo a adduser
        try:
            user=pwd.getpwnam(self.id)
            raise CgaUserExists("El usuario %s ya existe" % self.id)
        except KeyError:
            pass
        # El usuario no existe, podemos proceder.

        # Primero se cifra el password para pasarlo a adduser
        passwd = crypt(self.password.encode('utf8'), "00")
        uid=self._get_new_uid()

        # Cambio 7173: Cambio de los permisos de los usuarios locales.
        umask="077"


        #Creacion de usuarios local con el grupo igual que el nombre de usuario
        comando = "/usr/sbin/useradd -m -G {grupos} -p {password} -u {uid} -U -s /bin/bash -K UMASK={umask} {usuario}"
        comando = comando.format(   grupos=DEFAULTGROUPS,
                                    password=passwd,
                                    usuario=self.id,
                                    uid=uid,
                                    umask=umask)
        #print "Ejecutando '%s'" % comando
        
        (ret, salida)=commands.getstatusoutput(comando)
        comando=None
        if ret != 0:
            raise CgaUserCreationError("Error al crear el usuario: '%s'" % salida)


        comando='/usr/bin/chfn -f "'+ self.nombre +'"' + ' ' + self.id
        comando = comando.encode('utf8')

        (ret, salida)=commands.getstatusoutput(comando)
        comando=None
        if ret!=0:
            # Si el comando anterior falla tampoco pasa nada, es información optativa
            print "No se pudo establecer el GECOS: %s" % salida


        comando = ('echo "{username}:{password}" | chpasswd'.format(
                                    username=self.id,
                                    password=self.password))

        (ret, salida)=commands.getstatusoutput(comando)
        comando=None
        if ret!=0:
            print "Error al modificar la contraseña"



    def _add_to_list(self):
        usuario = os.environ["LOGNAME"]
        fecha=str(datetime.datetime.now())
        with open(CUSTOMUSERSFILE, "a") as f:
            f.write("%s:%s:%s\n" % (usuario, fecha, self.id))




if __name__=="__main__":
    c = CgaUser()
    c.save()
